@@ -300,11 +300,11 @@ while 1:
   skew = 0 
   arr = mysql(c, """
     SELECT session.sessnum, session.username, appname,
-    timeout - TIMESTAMPDIFF(SECOND, accesstime, NOW()) AS remaining
-    FROM session 
-    LEFT JOIN view ON session.sessnum = view.sessnum
-    WHERE viewid IS NULL AND 
-    timeout - TIMESTAMPDIFF(SECOND, accesstime, NOW()) < %d
+      timeout - TIMESTAMPDIFF(SECOND, accesstime, NOW()) AS remaining
+    FROM session LEFT JOIN view ON session.sessnum = view.sessnum
+    WHERE viewid IS NULL 
+    AND exechost != 'submit'
+    AND timeout - TIMESTAMPDIFF(SECOND, accesstime, NOW()) < %d
     ORDER BY remaining""" % (expire_interval))
 
   # Check if those sessions have jobs running
@@ -315,6 +315,9 @@ while 1:
   for sessnum, username, appname, session_remaining in arr:
     # Find the greatest remaining time for this session to stay alive
     max_remaining = int(session_remaining)
+    # short circuit other checks if this session will stay alive longer than interval to next check
+    if max_remaining > sleeptime:
+      continue
     # heartbeat is considered "stale" after 24 hours
     # remaining = heartbeat - NOW() + 86400
     job_arr = mysql(c, """SELECT 86400-TIMESTAMPDIFF(SECOND, heartbeat, NOW()) AS remaining
@@ -326,6 +329,8 @@ while 1:
         remaining = row[0]
         if int(remaining)  > max_remaining:
           max_remaining = int(remaining)
+      if max_remaining > sleeptime:
+        continue
 
     # How long should someone have to check the results of a simulation?  
     # people have 24 hours to check results after the end of the last job.  
@@ -351,14 +356,7 @@ while 1:
       time.sleep(1) # Do not kill things off too quickly.
       skew += 1
     elif max_remaining < sleeptime:
-      sleeptime = max_remaining
-    if False:
-      if max_remaining > session_remaining:
-        # fake access as an optimization to avoid processing the same data again
-        # could this affect statistics?  Alternative is to check less often. ( 5 min instead of 15 seconds)
-        mysql(c, """UPDATE session SET accesstime = DATE_ADD(NOW(), INTERVAL %d - timeout SECOND)
-        WHERE sessnum = %d""" % (max_remaining - skew, session))
-      
+      sleeptime = max_remaining      
 
   sleeptime -= skew
   if (sleeptime > 0):
